HealIncTimer = LibStub("AceAddon-3.0"):NewAddon("HealIncTimer", "AceEvent-3.0")

local healComm = LibStub:GetLibrary("LibHealComm-3.0", true)
local instanceType
local playerName = UnitName("player")
local incomingHeals
	
function HealIncTimer:OnInitialize()
	self.modules.Config:SetupDB()
	self.bars = self.modules.Bars:LoadVisual()
	
	self:ReloadVisual("watchFixedUnit")
	self:ReloadVisual("width")
	self:ReloadVisual("scale")
	self:ReloadVisual("showAnchor")
	self:ReloadVisual("collapseBars")
	self:ReloadVisual("growUp")
	self:ReloadVisual("barColor")
	self:ReloadVisual("bgColor")
	self:ReloadVisual("textColor")
	self:ReloadVisual("timerColor")
	self:ReloadVisual("enableGradient")
	self:ReloadVisual("leftTimerPosition")
	self:ReloadVisual("maxBars")
	
	--monitor for zone change
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "ZONE_CHANGED_NEW_AREA")
end

function HealIncTimer:OnEnable()
	local type = select(2, IsInInstance())
	if (not self.db.profile.inside[type]) then
		return
	end
	
	-- regist PLAYER_TARGET_CHANGED
	self:ReloadVisual("watchFixedUnit")

	-- register callbacks
	healComm.RegisterCallback(self, "HealComm_DirectHealStart")
	healComm.RegisterCallback(self, "HealComm_DirectHealStop")
	healComm.RegisterCallback(self, "HealComm_DirectHealDelayed")
end

function HealIncTimer:OnDisable()	
	-- unregist events
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
	
	-- unregister callbacks
	healComm.UnregisterCallback(self, "HealComm_DirectHealStart")
	healComm.UnregisterCallback(self, "HealComm_DirectHealStop")
	healComm.UnregisterCallback(self, "HealComm_DirectHealDelayed")
end

function HealIncTimer:Reload()
	self:OnDisable()

	-- Check to see if we should enable it
	local type = select(2, IsInInstance())
	if (self.db.profile.inside[type]) then
		self:OnEnable()
	end
end

--see if we should enable CombatTimer in this zone
function HealIncTimer:ZONE_CHANGED_NEW_AREA()
	local type = select(2, IsInInstance())

	if (type ~= instanceType) then
		-- Check if it's supposed to be enabled in this zone
		if (self.db.profile.inside[type]) then
			self:OnEnable()
		else
			self:OnDisable()
		end
	end
	
	instanceType = type
end

function HealIncTimer:PLAYER_TARGET_CHANGED()
	self.bars:ClearTimers()
	self:WatchDynamicTarget(true)
	for healerFullName, healData in pairs(incomingHeals) do
		self:HandleHeal(healerFullName, "START", healData.size, healData.targets)
	end
end

function HealIncTimer:HealComm_DirectHealStart(event, healerFullName, healSize, endTime, ...)
	if (not self.db.profile.watchFixedUnit) then
		local healData = {
			size = healSize,
			targets = ...,
		}
		incomingHeals[healerFullName] = healData
	end
	
	self:HandleHeal(healerFullName, "START", healSize, ...)
end

function HealIncTimer:HealComm_DirectHealStop(event, healerFullName, healSize, succeeded, ...)
	if (not self.db.profile.watchFixedUnit) then
		incomingHeals[healerFullName] = nil
	end
	
	--return if heal of player succeded
	if (succeeded) then
		return
	end

	self:HandleHeal(healerFullName, "STOP", nil, ...)
end

function HealIncTimer:HealComm_DirectHealDelayed(event, healerFullName, healSize, endTime, ...)
	self:HandleHeal(healerFullName, "DELAY", nil, ...)
end

function HealIncTimer:HandleHeal(healerFullName, type, healSize, ...)
	--return if we dont watch self heals
	local isOwnHeal = (healerFullName == playerName)
	if (isOwnHeal and not self.db.profile.watchSelfHeal) then
		return
	end

	--iterate through targets of heal
	for i = 1, select("#", ...) do
		local targetFullName = select(i, ...)
		if (self:StripServer(targetFullName) == self.db.profile.watchedUnit) then
			if (type == "START") then
				local spell, rank, _, icon, startTime, endTime = UnitCastingInfo(healerFullName)
				_, rank = string.match(rank, "(.-)% (.*)$")
				
				local text = self.db.profile.textMask
				text = string.gsub(text, "*spell", spell)
				text = string.gsub(text, "*rank", rank)
				text = string.gsub(text, "*player", self:StripServer(healerFullName))
				text = string.gsub(text, "*size", healSize)
				
				if (self.db.profile.announce) then
					local msg = self.db.profile.announceText
					msg = string.gsub(msg, "*spell", spell)
					msg = string.gsub(msg, "*rank", rank)
					msg = string.gsub(msg, "*player", self:StripServer(healerFullName))
					msg = string.gsub(msg, "*size", healSize)
					
					self:SendMessage(msg, self.db.profile.announceDest, self.db.profile.announceColor, icon)
				end
				
				self.bars:CreateTimer(healerFullName, text, startTime/1000, endTime/1000, icon)
			elseif (type == "DELAY") then
				self.bars:DelayTimer(healerFullName, select(6, UnitCastingInfo(healerFullName))/1000)
			elseif (type == "STOP") then
				self.bars:RemoveTimer(healerFullName)
			end
		end
	end
end

function HealIncTimer:UnitInRaid(unit)
	return (UnitInParty(unit) or UnitInRaid(unit) or UnitInBattleground(unit))
end

function HealIncTimer:WatchDynamicTarget()
	--if unit is not a player and unit is not in party/raid
	if (not UnitIsPlayer("target") and not HealIncTimer:UnitInRaid("target")) then
		self.db.profile.watchedUnit = ""
	else
		self.db.profile.watchedUnit = UnitName("target")
	end
	
	HealIncTimer:ReloadVisual("watch")
end

function HealIncTimer:WatchTarget()
	--return if unit is not a player
	if (not UnitIsPlayer("target")) then
		HealIncTimer:Print("Watched target must be a player")
		return
	end
	
	--return if unit is not in party/raid
	if (not HealIncTimer:UnitInRaid("target")) then
		HealIncTimer:Print("Watched target must be in the party/raid")
		return
	end
	
	self.db.profile.watchedUnit = UnitName("target")
	
	if (not self.db.profile.watchFixedUnit) then
		self.db.profile.watchFixedUnit = true
		HealIncTimer:ReloadVisual("watchFixedUnit")
	else
		HealIncTimer:ReloadVisual("watch")
	end
	
	HealIncTimer:Print("Watching "..self.db.profile.watchedUnit)
end

function HealIncTimer:ReloadVisual(arg, value)
	if (arg == "inside") then
		if( select(2, IsInInstance()) == value ) then
			if( self.db.profile[arg][value] ) then
				HealIncTimer:OnEnable()
			else
				HealIncTimer:OnDisable()
			end
		end
		
	elseif (arg == "watchFixedUnit") then
		if (self.db.profile.watchFixedUnit) then
			self:UnregisterEvent("PLAYER_TARGET_CHANGED")
			incomingHeals = nil
		else
			self:RegisterEvent("PLAYER_TARGET_CHANGED")
			incomingHeals = {}
		end
		self:ReloadVisual("watch")
		
	elseif (arg == "watch") then
		if (self.db.profile.watchedUnit == "") then
			self.bars.group:SetText("nothing watched")
		else
			self.bars.group:SetText(self.db.profile.watchedUnit)
		end
		
	elseif (arg == "width") then
		self.bars.group:SetWidth(self.db.profile.width)
		
	elseif (arg == "scale") then
		self.bars.group:SetScale(self.db.profile.scale)
		
	elseif (arg == "showAnchor") then
		self.bars.group:SetAnchorVisible(self.db.profile.showAnchor)
		
	elseif (arg == "enableGradient") then
		self.bars.group:EnableGradient(self.db.profile.enableGradient)
		
	elseif (arg == "growUp" or arg == "collapseBars") then
		self.bars.group:SetBarGrowth(self.db.profile.growUp and "UP" or "DOWN", self.db.profile.collapseBars and "LEFT" or "RIGHT")
		
	elseif (arg == "leftTimerPosition") then
		self.bars.group:SetTimerPosition(self.db.profile.leftTimerPosition and "LEFT" or "RIGHT")
		
	elseif (arg == "barColor") then
		self.bars.group:SetBarColor(self.db.profile.barColor.r, self.db.profile.barColor.g, self.db.profile.barColor.b)
		
	elseif (arg == "bgColor") then
		self.bars.group:SetBarBgColor(self.db.profile.bgColor.r, self.db.profile.bgColor.g, self.db.profile.bgColor.b)
		
	elseif (arg == "textColor") then
		self.bars.group:SetTextColor(self.db.profile.textColor.r, self.db.profile.textColor.g, self.db.profile.textColor.b)
		
	elseif (arg == "timerColor") then
		self.bars.group:SetTimerColor(self.db.profile.timerColor.r, self.db.profile.timerColor.g, self.db.profile.timerColor.b)
		
	elseif (arg == "maxBars") then
		self.bars.group:SetMaxBars(self.db.profile.maxBars)
	
	elseif (arg == "texture") then
		self.bars.group:SetTexture(self.media:Fetch(self.media.MediaType.STATUSBAR, self.db.profile.texture))
	
	end
end

-- Strips server name
function HealIncTimer:StripServer(text)
	local name, server = string.match(text, "(.-)%-(.*)$")
	if (not name and not server) then
		return text
	end
	
	return name
end

-- See if we should wrap an icon stuff around this
function HealIncTimer:WrapIcon(msg, dest, icon)
	if (not icon) then
		return msg
	end
	
	-- CT can be bigger due to more room
	local size = 12
	if (dest == "ct") then
		size = 18
	end

	return string.format("|T%s:0:0|t %s", icon, msg)
end

function HealIncTimer:SendMessage(msg, dest, color, icon)
	if (dest == "none") then
		return
	end
	
	-- We're ungrouped, so redirect it to ct
	if ((dest == "rw" or dest == "raid") and GetNumRaidMembers() == 0 and GetNumPartyMembers() == 0) then
		dest = "ct"
	-- We're grouped, in a raid and not leader or assist
	elseif (dest == "rw" and not IsRaidLeader() and not IsRaidOfficer() and GetNumRaidMembers() > 0) then
		dest = "party"
	end
	
	-- Strip out any () leftover from no name being given
	msg = string.trim(string.gsub(msg, "%(%)", ""))
		
	-- Chat frame
	if (tonumber(dest)) then
		local frame = getglobal("ChatFrame" .. dest) or DEFAULT_CHAT_FRAME
		frame:AddMessage("|cff33ff99HealIncTimer|r|cffffffff:|r " .. self:WrapIcon(msg, dest, icon), color.r, color.g, color.b)
	-- Raid warning announcement to raid/party
	elseif (dest == "rw") then
		SendChatMessage(msg, "RAID_WARNING")
	-- Party chat
	elseif (dest == "party") then
		SendChatMessage(msg, "PARTY")
	-- Raid chat
	elseif (dest == "raid") then
		SendChatMessage(msg, "RAID")
	-- Combat text
	elseif (dest == "ct") then
		self:CombatText(self:WrapIcon(msg, dest, icon), color)
	end
end

function HealIncTimer:CombatText(text, color)	
	-- SCT
	if (IsAddOnLoaded("sct")) then
		SCT:DisplayText(text, color, nil, "event", 1)
	-- MSBT
	elseif (IsAddOnLoaded("MikScrollingBattleText")) then
		MikSBT.DisplayMessage(text, MikSBT.DISPLAYTYPE_NOTIFICATION, true, color.r * 255, color.g * 255, color.b * 255)		
	-- Parrot
	elseif (IsAddOnLoaded("Parrot")) then
		Parrot:ShowMessage(text, nil, nil, color.r, color.g, color.b)
	-- Blizzard Combat Text
	elseif (IsAddOnLoaded("Blizzard_CombatText")) then
		-- Haven't cached the movement function yet
		if (not COMBAT_TEXT_SCROLL_FUNCTION) then
			CombatText_UpdateDisplayedMessages()
		end
		
		CombatText_AddMessage(text, COMBAT_TEXT_SCROLL_FUNCTION, color.r, color.g, color.b)
	end
end

function HealIncTimer:Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99HealIncTimer|r: " .. msg)
end
