if( not HealIncTimer ) then return end

local Config = HealIncTimer:NewModule("Config")
local addonVersion = tonumber(string.match(GetAddOnMetadata("HealIncTimer", "Version"), "(%d+)") or 1)

--common functions for options callbacks
local function get(info)
	local arg1, arg2 = string.split(".", info.arg)
	
	if (arg2) then
		return HealIncTimer.db.profile[arg1][arg2]
	else
		return HealIncTimer.db.profile[arg1]
	end
end

local function set(info, value)
	local arg1, arg2 = string.split(".", info.arg)
	
	if (arg2) then
		HealIncTimer.db.profile[arg1][arg2] = value
	else
		HealIncTimer.db.profile[arg1] = value
	end
	
	HealIncTimer:ReloadVisual(info.arg)
end

local function setMulti(info, value, state)
	local arg1, arg2 = string.split(".", info.arg)

	if (arg2) then
		HealIncTimer.db.profile[arg1][arg2][value] = state
	else
		HealIncTimer.db.profile[arg1][value] = state
	end

	HealIncTimer:ReloadVisual(info.arg, value)
end

local function setColor(info, r, g, b)
	set(info, {r = r, g = g, b = b})
end

local function getColor(info)
	local value = get(info)
	if( type(value) == "table" ) then
		return value.r, value.g, value.b
	end
	
	return value
end

local function slashHandler(option)
	if option == "menu" then
		InterfaceOptionsFrame_OpenToFrame("Heal Inc Timer")
	elseif option == "watch" then
		HealIncTimer:WatchTarget()
	else
		HealIncTimer:Print("Revision "..addonVersion)
		HealIncTimer:Print("/healinctimer watch")
		HealIncTimer:Print("/healinctimer menu")
	end
end

function Config:SetupDB()	
	HealIncTimer.media = LibStub:GetLibrary("LibSharedMedia-3.0")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "BantoBar", "Interface\\Addons\\HealIncTimer\\images\\banto")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Smooth", "Interface\\Addons\\HealIncTimer\\images\\smooth")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Perl", "Interface\\Addons\\HealIncTimer\\images\\perl")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Glaze", "Interface\\Addons\\HealIncTimer\\images\\glaze")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Charcoal", "Interface\\Addons\\HealIncTimer\\images\\Charcoal")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Otravi", "Interface\\Addons\\HealIncTimer\\images\\otravi")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "Striped", "Interface\\Addons\\HealIncTimer\\images\\striped")
	HealIncTimer.media:Register(HealIncTimer.media.MediaType.STATUSBAR, "LiteStep", "Interface\\Addons\\HealIncTimer\\images\\LiteStep")
	HealIncTimer.media.RegisterCallback(Config, "LibSharedMedia_Registered")
	
	local defaults = {
		profile = {
			inside = {["arena"] = true, ["pvp"] = true, ["none"] = true, ["party"] = true, ["raid"] = true},
			watchedUnit = UnitName("player"),
			watchFixedUnit = false,
			width = 180,
			scale = 1.0,
			showAnchor = true,
			texture = "Glaze",
			textMask = "*player - *spell (*rank) @*size",
			announce = false,
			announceColor = { r = 1.0, g = 1.0, b = 1.0 },
			announceDest = "1",
			announceText = "INC *player's *spell (*rank) @*size",
			enableGradient = true,
			barColor = { r = 0.0, g = 1.0, b = 0.0 },
			bgColor = { r = 0.0, g = 0.5, b = 0.5 },
			textColor = { r = 1.0, g = 1.0, b = 1.0 },
			timerColor = { r = 1.0, g = 1.0, b = 1.0 },
			collapseBars = false,
			growUp = false,
			leftTimerPosition = false,
			maxBars = 10,
		},
	}
	HealIncTimer.db = LibStub:GetLibrary("AceDB-3.0"):New("HealIncTimerDB", defaults)
	HealIncTimer.db.profile.version = addonVersion
	
	local enabledIn = {
		arena = "Arenas",
		pvp = "Battlegrounds",
		none = "Everywhere else",
		party = "Party instance",
		raid = "Raid instance",
	}
	
	local announceDest = {
		none = "None",
		ct = "Combat text",
		party = "Party",
		raid = "Raid",
		rw = "Raid warning",
		["1"] = "Chat frame #1",
		["2"] = "Chat frame #2",
		["3"] = "Chat frame #3",
		["4"] = "Chat frame #4",
		["5"] = "Chat frame #5",
		["6"] = "Chat frame #6",
		["7"] = "Chat frame #7",
	}
	
	local options = {}
	options.general = {
		type = "group",
		name = "Heal Inc Timer",
		get = get,
		set = set,
		args = {
			showAnchor = {
				order = 1,
				name = "Show anchor",
				desc = "Display timer anchors for moving around.",
				type = "toggle",
				arg = "showAnchor",
			},
			watchSelfHeal = {
				order = 2,
				name = "Watch my heal",
				type = "toggle",
				arg = "watchSelfHeal",
			},
			watchFixedUnit = {
				order = 3,
				name = "Watch fixed unit",
				type = "toggle",
				arg = "watchFixedUnit",
			},
			watchTarget = {
				order = 4,
				name = "Watch current target",
				type = "execute",
				func = function() HealIncTimer:WatchTarget() end,
			},
			announce = {
				order = 5,
				type = "group",
				inline = true,
				name = "Announcements",
				args = {
					enabled = {
						order = 0,
						type = "toggle",
						name = "Enable announcements",
						arg = "announce",
						width = "double",
					},
					color = {
						order = 1,
						type = "color",
						name = "Announcement color",
						set = setColor,
						get = getColor,
						arg = "announceColor",
						width = "double",
					},
					dest = {
						order = 2,
						type = "select",
						name = "Announce destination",
						values = announceDest,
						arg = "announceDest",
						width = "double",
					},
					text = {
						order = 3,
						name = "Announce message",
						type = "input",
						set = setVisualOption,
						arg = "announceText",
						width = "double",
					},
				},
			},
			inside = {
				order = 6,
				name = "Only enable inside",
				values = enabledIn,
				type = "multiselect",
				arg = "inside",
				set = setMulti,
			},
		}
	}
	
	options.bars = {
		type = "group",
		name = "Bars",
		get = get,
		set = set,
		args = {
			width = {
				order = 1,
				name = "Bar width",
				type = "range", isPercent = false,
				min = 20, max = 300, step = 1,
				arg = "width",
			},
			scale = {
				order = 2,
				name = "Bar scale",
				type = "range", isPercent = false,
				min = 0.5, max = 2, step = 0.05,
				arg = "scale",
			},
			textMask = {
				order = 3,
				name = "Bar text",
				type = "input",
				arg = "textMask",
			},
			texture = {
				order = 4,
				type = "select",
				name = "Bar texture",
				values = HealIncTimer.media:List('statusbar'),
				get=function(info)
					local mt = HealIncTimer.media:List('statusbar')
					for k,v in pairs(mt) do
						if v == HealIncTimer.db.profile.texture then
							return k
						end
					end
				end,
				set=function(info,value)
					local mt = HealIncTimer.media:List('statusbar')
					set({arg="texture"}, mt[value])
				end,
			},
			growUp = {
				order = 5,
				name = "Grow bars up",
				type = "toggle",
				arg = "growUp",
			},
			collapseBars = {
				order = 6,
				name = "Collapse bars",
				type = "toggle",
				arg = "collapseBars",
			},
			leftTimerPosition = {
				order = 7,
				name = "Left timer position",
				type = "toggle",
				arg = "leftTimerPosition",
				width = "double",
			},
			barColor = {
				order = 8,
				type = "color",
				name = "Bar color",
				set = setColor,
				get = getColor,
				arg = "barColor",
			},
			bgColor = {
				order = 10,
				type = "color",
				name = "Bar background color",
				set = setColor,
				get = getColor,
				arg = "bgColor",
			},
			textColor = {
				order = 11,
				type = "color",
				name = "Text color",
				set = setColor,
				get = getColor,
				arg = "textColor",
			},
			timerColor = {
				order = 12,
				type = "color",
				name = "Timer color",
				set = setColor,
				get = getColor,
				arg = "timerColor",
			},
			enableGradient = {
				order = 13,
				name = "Enable gradient bar color",
				type = "toggle",
				arg = "enableGradient",
				width = "double",
			},
			maxBars = {
				order = 14,
				name = "Maximum displayed bars",
				type = "range", isPercent = false,
				min = 1, max = 20, step = 1,
				arg = "maxBars",
			},
		}
	}
	
	local config = LibStub("AceConfigRegistry-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	
	config:RegisterOptionsTable("Heal Inc Timer", options.general)
	dialog:AddToBlizOptions("Heal Inc Timer", options.general.name)
	
	config:RegisterOptionsTable("Heal Inc Timer-Bars", options.bars)
	dialog:AddToBlizOptions("Heal Inc Timer-Bars", options.bars.name, "Heal Inc Timer")
	
	local console = LibStub("AceConsole-3.0")
	console:RegisterChatCommand("hinc", slashHandler)
	console:RegisterChatCommand("healinctimer", slashHandler)
end

function Config:LibSharedMedia_Registered(event, mediatype, key)
	if (mediatype == "statusbar" and key == HealIncTimer.db.profile.texture) then
		HealIncTimer:ReloadVisual("texture")
		HealIncTimer.media.UnregisterCallback(Config, "LibSharedMedia_Registered")
	end
end
